use serde::Deserialize;
use snafu::prelude::*;
use xdg::{BaseDirectories, BaseDirectoriesError};

use std::path::{Path, PathBuf};

use crate::cli::Cli;

#[derive(Deserialize, Debug)]
pub struct Config {
    pub qbittorrent: ClientConfig,
}

#[derive(Deserialize, Debug)]
pub struct ClientConfig {
    pub host: String,
    pub port: usize,
    pub login: String,
    pub password: String,
}

#[derive(Debug, Snafu)]
pub enum ConfigError {
    #[snafu(display("Invalid configuration file format at: {path}"))]
    Config { source: toml::de::Error, path: String },
    #[snafu(display("Failed to read configuration file {path}"))]
    IO { source: std::io::Error, path: String },
    #[snafu(display("Failed to look up $XDG_CONFIG_DIR"))]
    XDG { source: BaseDirectoriesError },
    #[snafu(display("Configuration file not found in $XDG_CONFIG_DIR/qbt/.qbt.toml"))]
    NotFound,
}

impl Config {
    pub fn default_path() -> Result<PathBuf, ConfigError> {
        let xdg_dirs = BaseDirectories::with_prefix("qbt").context(XDGSnafu)?;
        if let Some(f) = xdg_dirs.find_config_file(".qbt.toml") {
            Ok(f)
        } else {
            Err(ConfigError::NotFound)
        }
    }

    pub fn from_path<T: AsRef<Path>>(path: T) -> Result<Config, ConfigError> {
        let path_str = path.as_ref().to_str().expect("Unicode error");
        let config_str = std::fs::read_to_string(&path).context(IOSnafu { path: path_str })?;
        Ok(toml::from_str(&config_str).context(ConfigSnafu { path: path_str } )?)
    }

    pub fn from_default_path() -> Result<Config, ConfigError> {
        Self::from_path(&Self::default_path()?)
    }

    pub fn from_cli(cli: &Cli) -> Result<Config, ConfigError> {
        if let Some(cfg_path) = &cli.config {
            Self::from_path(cfg_path)
        } else {
            Self::from_default_path()
        }
    }

    pub fn format_host(&self) -> String {
        if self.qbittorrent.host.starts_with("http") {
            format!("{}:{}", self.qbittorrent.host, self.qbittorrent.port)
        } else {
            format!("http://{}:{}", self.qbittorrent.host, self.qbittorrent.port)
        }
    }
}
